{% extends "basecontrol.html" %}
{% from "helper.html" import expand_ctrl_props with context %}
{% block control %}
        <splunk-search-dropdown name="action.{{ mod_alert.short_name }}.param.{{ param.name }}" id="{{ mod_alert.short_name }}_{{ param.name }}"
{{ expand_ctrl_props(param) }} />
{% endblock %}
