import{k as t}from"./entry_page.js";import{d as o,E as s,a as e}from"./Dashboard.EnterpriseViewOnlyPreset.hOZR46Qg.js";import"./Search.BNBTqs0I.js";import"./ErrorBoundary.xJor3Ppq.js";const m=({dashboardDefinition:r})=>r?t.jsx(o.DashboardContextProvider,{preset:s,initialDefinition:r,children:t.jsx(e.DashboardCore,{width:"100%",height:"auto"})}):null;export{m as CustomDashboard,m as default};
