import{h as e,k as r}from"./entry_page.js";import{d as o,E as a,a as i}from"./Dashboard.EnterpriseViewOnlyPreset.hOZR46Qg.js";import{w as s,g as n}from"./Dashboard.utils.0p-tEVNs.js";import{F as p}from"./Dashboard.consts.BpCQqH5Z.js";import"./Search.BNBTqs0I.js";import"./ErrorBoundary.xJor3Ppq.js";import"./ArrowBroadUnderbarDown.wq6id2ca.js";const x=({dashboardDefinition:t})=>(e.useEffect(()=>(s('[data-input-id="overview_input"]',"#overview_main_label_viz"),()=>{}),[]),t?r.jsx(o.DashboardContextProvider,{preset:a,initialDefinition:t,featureFlags:p,children:r.jsx(i.DashboardCore,{width:"99%",height:"auto",actionMenus:n("overview")})}):null);export{x as OverviewDashboard,x as default};
